from pathlib import Path
import duckdb
import os
import subprocess
from core_pro.ultilities import make_dir


# path
path = Path.home() / 'Downloads/Data/Item_Matching_Test'
if str(Path.home()) != '/Users/kevinkhang':
    path = Path('/media/kevin/data_4t/Test')

# data
name = 'Description_Images'
query = f"""
select *
,concat('http://f.shopee.vn/file/', UNNEST(array_slice(string_split(images, ','), 1, 1))) image_url
from parquet_scan('{str(path / f'{name}.parquet')}')
order by item_id, images
"""
df = duckdb.sql(query).pl()
print(df.shape, df['item_name'].n_unique())

# download
path_parq = path / f'{name}_0.parquet'
df.write_parquet(path_parq)

path_image = path / 'download_img'
make_dir(path_image)
folder = path_image / f'img'
if not folder.exists():
    os.chdir(str(path_image))
    command = (
        f"img2dataset --url_list={str(path_parq)} "
        f"--output_folder=img_{name}/ "
        f"--processes_count=16 "
        f"--thread_count=32 "
        f"--image_size=224 "
        f"--output_format=files "
        f"--input_format=parquet "
        f"--url_col=image_url "
        f"--number_sample_per_shard=50000 "
    )
    subprocess.run(command, shell=True)



from pathlib import Path
from pydantic import BaseModel, Field, computed_field
import duckdb
import re
from time import perf_counter
from item_matching.func.utilities import make_dir, rm_all_folder
from item_matching.pipeline.build_index_and_query import BuildIndexAndQuery, ConfigQuery
from item_matching.pipeline.data_loading import DataEmbedding, ConfigEmbedding

import sys
from loguru import logger
logger.remove()
logger.add(sys.stdout, colorize=True, format='<level>{level}</level> | <cyan>{function}</cyan> | <level>{message}</level>')


class ModelInput(BaseModel):
    ROOT_PATH: Path = Field(default=None)
    PATH_Q: Path = Field(default=None)
    PATH_DB: Path = Field(default=None)
    MATCH_BY: str = Field(default='text') # image
    COL_CATEGORY: str = Field(default='')
    SHARD_SIZE: int = Field(default=1_500_000)
    QUERY_SIZE: int = Field(default=50_000)
    TOP_K: int = Field(default=10)
    MODE: str = Field(default='')

    @computed_field
    @property
    def path_result(self) -> Path:
        path_result = self.ROOT_PATH / f'result_match_{self.MATCH_BY}'
        make_dir(path_result)
        return path_result


class PipelineMatch:
    def __init__(self, record: ModelInput):
        self.record = record
        self.lst_category = []

    def category_chunking(self):
        # read query file to extract category
        query = f"""
        select distinct q_{self.record.COL_CATEGORY} as category 
        from read_parquet('{self.record.PATH_Q}')
        """
        self.lst_category = duckdb.sql(query).pl()['category'].to_list()

    def load_data(self, cat: str, mode: str = ''):
        file = self.record.model_dump().get(f'PATH_{mode.upper()}')
        query = f"""
        select * 
        from read_parquet('{file}') 
        where {mode}_{self.record.COL_CATEGORY} = '{cat}'
        """
        return duckdb.sql(query).pl()

    def run(self, export_type: str = 'parquet'):
        # extract category
        self.category_chunking()

        # run
        start = perf_counter()
        for idx, cat in enumerate(self.lst_category):
            # chunk checking
            chunk_db = self.load_data(cat, 'db')
            chunk_q = self.load_data(cat, 'q')

            print(
                f"🐋 Start matching by [{self.record.MATCH_BY}] cat: {cat} {idx}/{len(self.lst_category)} - "
                f"Database shape {chunk_db.shape}, Query shape {chunk_q.shape}"
            )

            if chunk_q.shape[0] == 0 or chunk_db.shape[0] == 0:
                print(f'Database/Query have no data')
                continue

            cat = re.sub('/', '', cat)
            file_name = self.record.path_result / f'{cat}.{export_type}'
            if file_name.exists():
                print(f'File already exists: {file_name}')
                continue

            # embeddings
            input_data = self.record.model_copy(update={'MODE': 'db'}).model_dump()
            DataEmbedding(config_input=ConfigEmbedding(**input_data)).load(data=chunk_db)

            input_data = self.record.model_copy(update={'MODE': 'q'}).model_dump()
            DataEmbedding(config_input=ConfigEmbedding(**input_data)).load(data=chunk_q)

            # index and query
            build = BuildIndexAndQuery(config=ConfigQuery(**self.record.model_dump()))
            build.build()
            df_match = build.query()

            # export
            if export_type == 'parquet':
                df_match.write_parquet(file_name)
            else:
                df_match.write_csv(file_name)

            for name in ['index', 'result', 'db_array', 'db_ds', 'q_array', 'q_ds']:
                rm_all_folder(self.record.ROOT_PATH / name)

        time_perf = perf_counter() - start
        print(f'🐋 Your files are ready, please find here: {self.record.path_result}')
        return {'time_perf': time_perf, 'path_result': self.record.path_result}



from pathlib import Path
import polars as pl
import matplotlib.pyplot as plt
from PIL import Image
import numpy as np


class Draw:
    def __init__(self):
        pass

    @staticmethod
    def plot_one_image(file_path: str, fig_size: tuple = (7, 3)):
        fig, ax = plt.subplots(1, 1, figsize=fig_size)
        ax.imshow(Image.open(file_path))
        fig.tight_layout()

    @staticmethod
    def plot_multiple_images(file_paths: list, fig_size: tuple = (20, 10)):
        # layout
        num_images, ncol = min(len(file_paths), 5), 5
        nrow = int(np.ceil(num_images / ncol))
        fig, axes = plt.subplots(nrow, ncol, figsize=fig_size)
        axes = axes.flatten()

        # plot
        for i, image in enumerate(file_paths):
            axes[i].imshow(Image.open(image))
            axes[i].axis('off')

        # remove axes
        for _ in range(num_images, ncol * nrow, 1):
            axes[_].remove()
        fig.tight_layout()


def rm_all_folder(path: Path) -> None:
    for child in path.glob('*'):
        if child.is_file():
            child.unlink()
        else:
            rm_all_folder(child)
    path.rmdir()


def make_dir(folder_name: str | Path) -> None:
    """Make a directory if it doesn't exist"""
    if isinstance(folder_name, str):
        folder_name = Path(folder_name)
    if not folder_name.exists():
        folder_name.mkdir(parents=True, exist_ok=True)


def clean_check(data: pl.DataFrame, pct: bool = False, verbose: bool = False) -> dict:
    null_check_dict = data.null_count().to_dicts()[0].items()
    null_check = {i: v for i, v in null_check_dict if v > 0}
    if pct:
        null_check = {i: v / data.shape[0] for i, v in null_check_dict if v > 0}

    if verbose:
        print(f"- Null columns: {null_check}")
    dict_ = {
        'null': null_check,
    }
    return dict_



from core_pro import DataPipeLine
from pathlib import Path


path = Path.home() / 'Downloads/Data/Item_Matching_Test'
if str(Path.home()) != '/Users/kevinkhang':
    path = Path('/media/kevin/data_4t/Test')

file = path / 'Description_Images.parquet'
sql = open('./data.sql').read()
df = DataPipeLine(sql).run_presto_to_df(save_path=file)



import matplotlib.pyplot as plt
from PIL import Image
import numpy as np
from pathlib import Path
import textwrap
import polars as pl
import duckdb
import sys
import re
import emoji
from tqdm import tqdm
from loguru import logger
logger.remove()
logger.add(sys.stdout, colorize=True, format='<level>{level}</level> | <cyan>{function}</cyan> | <level>{message}</level>')


class Draw:
    def __init__(self):
        pass

    @staticmethod
    def plot_one_image(file_path: Path, title: str = None, fig_size: tuple = (7, 3)):
        fig, ax = plt.subplots(1, 1, figsize=fig_size)
        ax.imshow(Image.open(file_path))
        ax.axis('off')
        if title:
            ax.set_title(textwrap.fill(title, width=30, break_long_words=True), wrap=True, fontsize=10, ha='center')
        fig.tight_layout()
        fig.show()

    @staticmethod
    def plot_one_image_long_caption(file_path: Path, caption: str, fig_size: tuple = (12, 5), vertical: bool = False):
        nrows, ncols = 1, 2
        if vertical:
            nrows, ncols = 2, 1
            fig_size = (5, 12)

        fig, (ax1, ax2) = plt.subplots(nrows, ncols, figsize=fig_size)
        ax1.imshow(Image.open(file_path))
        ax1.axis('off')
        ax2.text(0, 0.5, caption, va='center', ha='left', wrap=True)
        ax2.axis('off')
        fig.tight_layout()
        fig.show()

    @staticmethod
    def plot_multiple_images(file_paths: list, title: list = None, fig_size: tuple = (20, 10)):
        # layout
        num_images, ncol = len(file_paths), min(len(file_paths), 3)
        nrow = int(np.ceil(num_images / ncol))
        fig, axes = plt.subplots(nrow, ncol, figsize=fig_size)
        axes = axes.flatten()

        # plot
        for i, (image, t) in enumerate(zip(file_paths, title)):
            axes[i].imshow(Image.open(image))
            axes[i].axis('off')
            if title:
                axes[i].set_title(textwrap.fill(t, width=40, break_long_words=True), wrap=True, fontsize=10, ha='center')
        # remove axes
        for _ in range(num_images, ncol * nrow, 1):
            axes[_].remove()
        fig.tight_layout()


class PipelineText:
    def __init__(self, mode: str = ''):
        self.mode = mode

    @staticmethod
    def remove_text_between_emojis(text):
        # regex pattern to match emojis
        emoji_pattern = re.compile(
            "["
            u"\U0001F600-\U0001F64F"  # emoticons
            u"\U0001F300-\U0001F5FF"  # symbols & pictographs
            u"\U0001F680-\U0001F6FF"  # transport & map symbols
            u"\U0001F1E0-\U0001F1FF"  # flags (iOS)
            u"\U00002702-\U000027B0"
            u"\U000024C2-\U0001F251"
            "]+",
            flags=re.UNICODE
        )
        # find all emojis in the text
        emojis = emoji_pattern.findall(text)
        # if there are less than 2 emojis, return the original text
        if len(emojis) < 2:
            return text
        else:
            regex = f"[{emojis[0]}].*?[{emojis[1]}]"
            return re.sub(regex, "", text)

    @staticmethod
    def clean_text_pipeline(text: str) -> str:
        regex = r"[\(\[\<\"\|].*?[\)\]\>\"\|]"
        text = str(text).lower().strip()
        text = PipelineText.remove_text_between_emojis(text)
        text = emoji.replace_emoji(text, ' ')
        text = re.sub(regex, ' ', text)
        text = re.sub(r'\-|\_|\*', ' ', text)
        text = re.sub(r'\s+', ' ', text)
        return text.rstrip('.').strip()

    @staticmethod
    def clean_text(data: pl.DataFrame, col: str = 'item_name') -> pl.DataFrame:
        lst = [PipelineText.clean_text_pipeline(str(x)) for x in tqdm(data[col].to_list(), desc='[Pipeline] Clean Text')]
        return data.with_columns(pl.Series(name=f'{col}_clean', values=lst))

    def run(self, data, key_col: list = None):
        # load data
        query = f"""select * from data"""
        df = duckdb.sql(query).pl()
        logger.info(f'[Data] Base Data {self.mode}: {df.shape}')

        df = (
            df
            .pipe(PipelineText.clean_text)
            .drop_nulls(subset=key_col)
        )
        logger.info(f'[Data] Join Data {self.mode}: {df.shape}')
        return df
